import pathlib
from typing import Iterable
import bioplumber.configs as configs
import requests
import rich
from rich.progress import Progress
REFERENCE_GENOMES={
    "homo_sapiens":"http://igenomes.illumina.com.s3-website-us-east-1.amazonaws.com/Homo_sapiens/UCSC/hg38/Homo_sapiens_UCSC_hg38.tar.gz",
    "PhiX":"https://webdata.illumina.com/downloads/productfiles/igenomes/phix/PhiX_Illumina_RTA.tar.gz",
    "Mus musculus (Mouse)":"http://igenomes.illumina.com.s3-website-us-east-1.amazonaws.com/Mus_musculus/Ensembl/GRCm38/Mus_musculus_Ensembl_GRCm38.tar.gz"
}
def group_files(path:str,
                separator:str="_",
                group_on:Iterable[int]=[0,1],
                extension:str="fastq.gz")->dict[int,list[str]]:
    
    """This function groups files based on their names.
    The files are grouped based on the separator and the group_on.
    for example if the files are named as follows:
    
    sample1_1.fastq.gz
    sample1_2.fastq.gz
    sample2_1.fastq.gz
    sample2_2.fastq.gz
    
    The function will group the files as follows:
    separator: "_"
    group_on: [0]
    This will output:
    {
        1: ["sample1_1.fastq.gz","sample1_2.fastq.gz"],
        2: ["sample2_1.fastq.gz","sample2_2.fastq.gz"]
    }
    
    Same files with the following parameters:
    separator: "_"
    group_on: [1]
    This will output:
    {
        1: ["sample1_1.fastq.gz","sample2_1.fastq.gz"],
        2: ["sample1_2.fastq.gz","sample2_2.fastq.gz"]
    }
    NOTE: Indeces for group_on are 0-based
    
    Args:
        path (str): The path to the files
        separator (str): The separator to use to split the file names
        group_on (Iterable[int]): The index of the group to use after splitting the file names
        extension (str): The extension of the files
    
    Returns:
        dict[int,list]: A dictionary with the group number as the key and the list of files as the value
    """
    path = pathlib.Path(path)
    all_files = path.rglob(f"*{extension}")
    group_files={}
    code_map={}    
    for file in all_files:
        file_name = file.name.replace(extension,"")
        file_parts = file_name.split(separator)
        code="".join([file_parts[i] for i in group_on])
    
        group_files.setdefault(code_map.setdefault(code,len(code_map)+1),[]).append(str(file.absolute()))
    
    for key in group_files:
        group_files[key].sort()
        
    return group_files

    

        
def cat_files_(files:Iterable[str],
               output_name:str,
               configs:configs.Configs,
               container:str="none",
               **kwargs)->str:
    """this function ouputs a command to use cat to concatenate files provided in the input
    
    Args:
        files (Iterable[str]): A list of file addresses to concatenate
        output_name (str): The name of the output file
        configs (configs.Configs): The configurationobjet to use
        container (str): The container to use to run the command: "none","singularity","docker"
    
    Returns:
        str: The path to the concatenated file
    """
    parents=[pathlib.Path(file).parent for file in files]
    if len(set(parents))>1:
        raise ValueError("All files should be in the same directory")

    output_path = pathlib.Path(parents[0]) / output_name
    if container=="none":
        cat_command = f"cat {' '.join(files)} > {output_path.absolute()}"
        for key,value in kwargs.items():
            cat_command = cat_command + f" --{key} {value}"
    
    
    elif container=="docker":
        mapfiles=" ".join([f"-v {str(pathlib.Path(file).absolute())}:{str(pathlib.Path(file).absolute())}" for file in files])
        cmd= " ".join([f"{str(pathlib.Path(file).absolute())}" for file in files])
        cat_command = f"docker run {mapfiles} {configs.docker_container} cat {cmd} > {str(output_path.absolute())}"
        for key,value in kwargs.items():
            cat_command = cat_command + f" --{key} {value}"
    
    
    elif container=="singularity":
        mapfiles=",".join([f"{str(pathlib.Path(file).absolute())}:{str(pathlib.Path(file).absolute())}" for file in files])
        cmd= " ".join([f"{str(pathlib.Path(file).absolute())}" for file in files])
        cat_command = f"singularity exec --bind {mapfiles} {configs.singularity_container} cat {cmd} > {str(output_path.absolute())}"
        for key,value in kwargs.items():
            cat_command = cat_command + f" --{key} {value}"
    
    return cat_command



def download_url(url:str, output_path:str)->None:
    """This function downloads a file from a url and saves it to the output path
    
    Args:
        url (str): The url to download the file from
        output_path (str): The path to save the downloaded file
    
    Returns:
        None
    """    
    r = requests.get(url, allow_redirects=True,stream=True,timeout=10)
     
    total_size = int(r.headers.get('content-length', 0))
    block_size = total_size // 1000 + 1  # Progress bar updates every 1%
    if not pathlib.Path(output_path).parent.exists():
        pathlib.Path(output_path).parent.mkdir(parents=True, exist_ok=True)
    with open(output_path, 'wb') as f:
        with Progress() as progress:
            task = progress.add_task("[cyan]Downloading...", total=total_size)
            for data in r.iter_content(block_size):
                f.write(data)
                progress.update(task, advance=len(data))
    
    
        
        


from textual.app import App
from textual.widget import Widget
from textual.widgets import Header, Footer, Tabs,Tab, TextArea, Button,DataTable
from textual.containers import Container,Horizontal,Vertical
from bioplumber import (configs,
                        bining,
                        files,
                        qc,
                        assemble,
                        slurm,
                        alignment)

import json
import os

SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))

def main():
    app = Bioplumber()
    app.run()


class EditableFileViewer(Container):
    """Widget to edit and save the contents of a text file."""

    def __init__(self, file_path: str, **kwargs):
        super().__init__(**kwargs)
        self.file_path = file_path
        self.text_area = TextArea(id="slurm_editor")  # Editable area
        self.save_button = Button("Save", id="save_button")
        
    def on_mount(self):
        """Load the file content into the text area."""
        self.mount(self.text_area, self.save_button)

        try:
            with open(self.file_path, "r") as file:
                content = file.read()
            self.text_area.text = content
        except Exception as e:
            self.text_area.text = f"Error loading file: {e}"

    def on_button_pressed(self, event: Button.Pressed):
        """Handle save button click."""
        if event.button.id == "save_button":
            try:
                with open(self.file_path, "w") as file:
                    file.write(self.text_area.text)
                self.text_area.insert("File saved successfully!\n",(0,0),  maintain_selection_offset=False)
            except Exception as e:
                self.text_area.insert( f"Error saving file: {e}\n",(0,0), maintain_selection_offset=False)
                

class SlurmManager(Container):
    
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        
    def on_mount(self):
        try:
            data=slurm.query_squeue()
            table=DataTable()
            table.add_columns(*[i for i in data.keys()])
            table.add_rows(*list(zip(*data.values())))
            yield table
        
            
        except Exception:
            yield TextArea(text="Make sure you have access slurm")  
    
class TabManager(Tabs):
    
    def compose(self):
        yield Tabs(
            Tab("Script Generator", id="sg"),
            Tab("Operation", id="op"),
            Tab("Job monitor", id="jm"),
            Tab("Slurm template", id="st"),
            id="all_tabs"
        )

class Bioplumber(App):
    CSS_PATH = "tui_css.tcss"
    BINDINGS=[
        ("d", "toggle_dark","Toggle dark mode")
    ]
    def compose(self):
        
        yield Header(show_clock=True)
        yield TabManager()
        yield Container(id="tab_contents")
        yield Footer()
 
    def on_mount(self):
        """Load initial content for the first tab."""
        self.load_tab_content("sg")
        
    def action_toggle_dark(self) -> None:
        """An action to toggle dark mode."""
        self.theme = (
            "textual-dark" if self.theme == "textual-light" else "textual-light"
        )
    
    def on_tabs_tab_activated(self, event: Tabs.TabActivated) -> None:
        """Handle tab activation events."""
        tab_id = event.tab.id
        self.load_tab_content(tab_id)
        
    def load_tab_content(self, tab_id: str):
        """Dynamically load content based on the selected tab."""
        container = self.query_one("#tab_contents")
        container.remove_children()
        print(tab_id)
        if tab_id == "st":
            # Add the editable file viewer content
            container.mount(EditableFileViewer(os.path.join(SCRIPT_DIR,"slurm_template.txt")))  # Replace with your file path
        elif tab_id == "jm":
            container.mount(SlurmManager())
            
        else:
        # Add placeholder content for other tabs
            container.mount(TextArea(text=f"Content for {tab_id} tab."))



if __name__ == "__main__":
    main()